#ifndef _k_means_H_
#define _k_means_H_
enum {
	k_means_ERROR = -1,
	k_means_SUCCESS,
};
/*
 * fields of struct k_means_context
 * k : number of means
 * dim: dimension of data
 * weight : array of the weights k-th mean for viewed data
 * mean   : array of means
 * dist   : maximal Euclid distance in claster
 */
struct k_means_context {
	int k, dim;
	double *weight, *means, *dists;
};

struct k_means_context * k_means_new();
void k_means_init_(struct k_means_context *ctx);
void k_means_free(struct k_means_context **ctx);
void k_means_clear(struct k_means_context *ctx);

/* Set number of mean. */
void k_means_set_nmean(struct k_means_context *ctx, int nmean);
/* Get number of mean. */
int k_means_get_nmean(struct k_means_context *ctx);

/* Set fitted data item dimension. */
void k_means_set_dimension(struct k_means_context *ctx, int dim);
/* Get fitted data item dimension. */
int k_means_get_dimension(struct k_means_context *ctx);

/* Set error value for end of iteration process. */
void k_means_set_error(struct k_means_context *ctx, double error);
/* Get error value for end of iteration process. */
double k_means_get_error(struct k_means_context *ctx);

/* Set n-th mean component weight. */
void k_means_set_nth_weight(struct k_means_context *ctx, int nth, double weight);
/* Get n-th mean component weight. */
double k_means_get_nth_weight(struct k_means_context *ctx, int nth);

/* Set n-th mean. */
void k_means_set_nth_mean(struct k_means_context *ctx, int nth, const double *mu);
/* Get n-th mean. */
void k_means_get_nth_mean(const struct k_means_context *ctx, int nth, double *mu);

/* Set n-th distance array. */
void k_means_set_nth_distance(struct k_means_context *ctx, int nth, const double *dist);
/* Get n-th distance array. */
void k_means_get_nth_distance(const struct k_means_context *ctx, int nth, double *cmx);

/*
 * Converge the k-means-algorithm, repeating k-means-steps until logarithm of the
 * likelihood function stops to change. Threshold of the change is given by epsilon.
 */
int k_means_converge(struct k_means_context *ctx, double *x, int nx, double epsilon);

/**
 * simple and slow k_mean algorithm
 */
int k_means(double *means, int nmeans, double *x, int nx, int dim, double epsilon);

/*
 * Determine if a sample x represents model or not.
 * return: k_means_SUCCESS - if x represents model and data is valid
 *         k_means_ERROR - else
 */
int k_means_sample_to_match_model(struct k_means_context *ctx, double *x);

/*
 * Update k_means_context using new sample data
 */
void k_means_update(struct k_means_context *ctx, double *x);

/**
 * @function
 * Reset k_means_context
 * sets the identity covariance matrix
 * sets weights to 1 / number of gaussians
 * sets means of the random from data (if data != NULL)
 * @param ctx       : pointer to struct k_means_context
 * @param data      : sample for reset model
 * @param data_size : data size
 * @note
 * if data == NULL fills the means with zeros
 */
void k_means_reset(struct k_means_context *ctx, double *data, int data_size);
#endif //_k_means_H_
